class_name GrunkSplatter extends GPUParticles3D
## Splatter effect with adjustable volume

const BASE_VOLUME := -12.0
const SCALE_FACTOR := 4.0

const SCENE := preload("res://src/effects/grunk_splatter/grunk_splatter.tscn")

@export var effect_scale := 1.0

@onready var splatter_sfx: AudioStreamPlayer3D = %SplatterSFX
@onready var sub_splatter: GPUParticles3D = %SubSplatter


# Called when the node enters the scene tree for the first time.
func _ready() -> void:
	splatter_sfx.volume_db = BASE_VOLUME + SCALE_FACTOR * log(effect_scale) / log(2)
	scale = Vector3.ONE * effect_scale

	# Scale particles themselves
	_scale_particles(self)
	_scale_particles(sub_splatter)


func _scale_particles(emitter: GPUParticles3D) -> void:
	var mat := emitter.process_material as ParticleProcessMaterial
	mat.scale_max = effect_scale
	mat.scale_min = effect_scale


static func build(_effect_scale: float = 1.0) -> GrunkSplatter:
	var instance: GrunkSplatter = SCENE.instantiate()
	instance.effect_scale = _effect_scale
	return instance
