class_name MP3Controller extends Control
## UI controller for the MP3 player

signal pause
signal resume
signal select_track(track_index: int)

var selecting := true

@onready var play_label: Label = %PlayLabel
@onready var pause_label: Label = %PauseLabel
@onready var retro_selector_list: RetroSelectorList = %RetroSelectorList
@onready var transition_player: AnimationPlayer = %TransitionPlayer


func set_track_list(track_list: Array[String]) -> void:
	retro_selector_list.options = track_list
	retro_selector_list.rebuild_list()


func set_paused(paused: bool) -> void:
	play_label.visible = not paused
	pause_label.visible = paused

	if paused:
		pause.emit()
	else:
		resume.emit()


func is_paused() -> bool:
	return pause_label.visible


## One of two input buttons to the device
func select() -> void:
	if transition_player.is_playing():
		return

	if selecting:
		# If we're in track selection, play the selected track
		selecting = false
		transition_player.play("to_player")
	else:
		# If we're in player mode, toggle pause/play
		set_paused(not is_paused())


func _emit_select_track() -> void:
	select_track.emit(retro_selector_list.selection_idx)
	set_paused(false)


## The other input button to the device
func cancel() -> void:
	if transition_player.is_playing():
		return

	if selecting:
		# If we're in track selection, advance selection to the next track
		retro_selector_list.selection_idx += 1
	else:
		# If we're in player mode, pause playback and go to track selection
		set_paused(true)
		selecting = true
		transition_player.play("to_selector")
