@tool
extends GunkNode

const BASE_DURABILITY := 3.0
const BASE_VALUE := 2000.0
const MIN_CHITTER_INTERVAL := 4.0

@export_category("Editor Tools")
@export var randomize_rotation: bool:
	set(v):
		if v:
			rotation = Vector3(randf_range(0, TAU), randf_range(0, TAU), randf_range(0, TAU))
@export var apply_scale: float:
	set(v):
		if v > 0.001:
			scale = Vector3(v, v, v)
			durability = BASE_DURABILITY * v
			value = BASE_VALUE * v

@export_category("Jitter")
@export var jitter_scale_factor := 0.05
@export var jitter_inflation_factor := 1.0

@export_category("Chatter")
@export var chitter: bool = false
@export var chitter_time_mean := 60.0
@export var chitter_time_st_dev := 30.0

@onready var mesh_instance: MeshInstance3D = %MeshInstance3D

@onready var chitter_sfx: AudioStreamPlayer3D = %ChitterSFX
@onready var chitter_timer: Timer = %ChitterTimer


func _ready() -> void:
	if Engine.is_editor_hint():
		return

	if chitter:
		start_chitter_timer()


func _process(delta: float) -> void:
	if Engine.is_editor_hint():
		return

	super._process(delta)
	var shader: ShaderMaterial = mesh_instance.get_surface_override_material(0)
	var damage := pct_damage()
	shader.set_shader_parameter("jitter_time_scale", pow(damage * jitter_scale_factor, 1.2))
	shader.set_shader_parameter("vertex_inflation", pow(damage * jitter_inflation_factor, 3))


func start_chitter_timer() -> void:
	var interval := maxf(MIN_CHITTER_INTERVAL, randfn(chitter_time_mean, chitter_time_st_dev))
	chitter_timer.start(interval)


func _on_chitter_timer_timeout() -> void:
	chitter_sfx.play()
	start_chitter_timer()
